/*         mList.c                    2009-01-27   Reiter Franz.

GTK-MultiColumnListfunctions 
    using gtk_tree_model gtk_tree_view gtk_list_store

-----------------------------------------------------
Modifications:

-----------------------------------------------------
*//*!
\file ../gtk/mList.c
\ingroup grp_gtk
\code

=====================================================
Liste_Funktionen_Beginn:

GUI_mList__            create simple List
GUI_mList_add          append Row to List
GUI_mList_lNr          query nr of lines in List
GUI_mList_read_iter    get text of row from iter
GUI_mList_read_rowNr   get text of row nr lNr
GUI_mList_DelRow       delete row lNr
GUI_mList_set_cursor   select row nr lNr
GUI_mList_iter2rowNr   get lineNumber from iter
GUI_mList_rowNr2iter   get iter from lineNumber
GUI_mList_modify_rowNr modify text of row lNr
GUI_mList_get_sel_nr   get lineNr of selected line

Liste_Funktionen_Ende:
=====================================================

- usage examples:

static GUIobj_mList    w_lst1;

  GUI_mList__ (box0, &w_lst1, &w_tree1, "colTit", 1, NULL,NULL,NULL);
  GUI_mList_add (&w_lst1, "first line");
  ..
  gtk_widget_grab_focus (GTK_WIDGET(w_lst1.tree));   // focus on list
  ..
  gtk_widget_set_sensitive (w_lst1.tree, FALSE);     // disable List

- see also:
  ../gtk/tst_mList.c


\endcode *//*----------------------------------------

*/


#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <gtk/gtk.h>

#include "../gtk/ut_gtk.h"       // GUIobj_mList




 
//================================================================
  int GUI_mList_add (GUIobj_mList *wLst, char *txt) {
//================================================================
/// append Row to List

  GtkTreeIter  iter;

  gtk_list_store_append ((GtkListStore*)wLst->list, &iter);

  gtk_list_store_set ((GtkListStore*)wLst->list, &iter, 0, txt, -1);


  return 0;

}



//================================================================
  int GUI_mList_modify_rowNr (GUIobj_mList *wLst, char *txt, int lNr) {
//================================================================
/// \code
/// modify text of row lNr
/// see also jb_list_load
/// \endcode

  GtkTreeIter  iter;

  GUI_mList_rowNr2iter (&iter, wLst, lNr);

  gtk_list_store_set ((GtkListStore*)wLst->list, &iter, 0, txt, -1);

 
  return 0;

}


//================================================================
  int GUI_mList_read_rowNr (char *outTxt, GUIobj_mList *wLst, int lNr) {
//================================================================
     
  int          irc;
  GtkTreeIter  iter;


  // get text of first row
  irc = GUI_mList_rowNr2iter (&iter, wLst, lNr);
  if(irc < 0) return -1;

  GUI_mList_read_iter (outTxt, wLst, &iter);

  return 0;

}


//================================================================
  int GUI_mList_rowNr2iter (GtkTreeIter *iter, GUIobj_mList *wLst, int lNr) {
//================================================================
/// \code
/// get iter from rownumber
///   lNr    (0 = first)
/// \endcode
 
  gboolean     valid;

  // printf("GUI_mList_rowNr2iter %d\n",lNr);


  // Get the first iter in the list 
  // valid = gtk_tree_model_get_iter_first (GTK_TREE_MODEL(wLst), &iter);
  valid = gtk_tree_model_iter_nth_child (GTK_TREE_MODEL(wLst->list),
                                         iter, NULL, lNr);

  if(valid == (gboolean)NULL) {
    printf("***** GUI_mList_rowNr2iter E001\n");
    return -1;
  }


  return 0;

}


//================================================================
  int GUI_mList_iter2rowNr (GUIobj_mList *wLst, GtkTreeIter *iter) {
//================================================================
/// get lineNumber from iter


  int          ii;
  gchar        *txt1;


  // get mcl_rNr = nr of selected row; first = 0
  txt1 = gtk_tree_model_get_string_from_iter ((GtkTreeModel*)wLst->list, iter);
  ii = atoi(txt1);
  g_free (txt1);

  // printf(" _string_from_iter %d\n",ii);

  return ii;

}
 

//================================================================
  int GUI_mList_set_cursor (GUIobj_mList *wLst, int lNr) {
//================================================================
/// \code
/// select row nr lNr         First line = 0
/// GUI_update__ ();          // force Update
/// \endcode

  char         cBuf1[16];
  GtkTreeIter  iter;
  GtkTreePath  *tpt;

  // printf("GUI_mList_set_cursor %d\n",lNr);

  // select next line ..
  sprintf(cBuf1, "%d:0",lNr);  // create "path"


  gtk_tree_model_get_iter_from_string ((GtkTreeModel*)wLst->list, &iter, cBuf1);
  tpt = gtk_tree_model_get_path ((GtkTreeModel*)wLst->list, &iter);
  gtk_tree_view_set_cursor ((GtkTreeView*)wLst->tree, tpt, NULL, FALSE);
  gtk_tree_path_free (tpt);

  return 0;

}
 

//================================================================
  int GUI_mList_DelRow (GUIobj_mList *wLst, int lNr) {
//================================================================
/// \code
/// delete row lNr
///   lNr     rewNumber; 0 = first row; -1 = delete all rows !
/// \endcode

  int          iCol, ii;
  gchar        *txt1;
  gboolean     valid;
  GtkTreeIter  iter;


  // remove all rows
  if(lNr < 0) {
    gtk_list_store_clear ((GtkListStore*)wLst->list);
    return 0;
  }


  // Get the first iter in the list 
  // valid = gtk_tree_model_get_iter_first (GTK_TREE_MODEL(wLst), &iter);

  // get row # lNr
  valid = gtk_tree_model_iter_nth_child (GTK_TREE_MODEL(wLst->list),
                                         &iter, NULL, lNr);
  if(valid == (gboolean)NULL) {
    printf("***** GUI_mList_DelRow E001\n");
    return -1;
  }

  // valid = gtk_list_store_remove (GTK_TREE_MODEL(wLst), &iter);
  valid = gtk_list_store_remove ((GtkListStore*)wLst->list, &iter);
  if(valid == (gboolean)NULL) {
    printf("***** GUI_mList_DelRow E002\n");
    return -1;
  }


  return 0;

}


//================================================================
  int GUI_mList_read_iter (char *txtOut, GUIobj_mList *wLst, GtkTreeIter *iter) {
//================================================================
/// \code
/// get text if row nr lNr
///   lNr    (0 = first)
/// \endcode
 
  int          iCol;
  gchar        *txt1;
  gboolean     valid;
  // GtkTreeIter  iter;

  printf("GUI_mList_read_iter \n");


/*
  // Get the first iter in the list 
  // valid = gtk_tree_model_get_iter_first (GTK_TREE_MODEL(wLst), &iter);
  valid = gtk_tree_model_iter_nth_child (GTK_TREE_MODEL(wLst),
                                         &iter, NULL, lNr);

  if(valid == NULL) {
    printf("***** GUI_mList_read_iter E001\n");
    return -1;
  }
*/
  iCol = 0;
  gtk_tree_model_get (GTK_TREE_MODEL(wLst->list), iter, 
                        iCol, &txt1, -1); // access Spalte 0
  strcpy(txtOut, txt1);

  g_free (txt1);

  printf("GUI_mList_read_iter |%s|\n",txtOut);

  return 0;

}


//================================================================
  int  GUI_mList_lNr (GUIobj_mList *wLst) {
//================================================================
/// query nr of lines in List

  int  iNr;

  // if iter is NULL, then the number of toplevel nodes is returned.
  iNr = gtk_tree_model_iter_n_children (GTK_TREE_MODEL(wLst->list), NULL);

    printf(" ex GUI_mList_lNr %d\n",iNr);

  return iNr;

}


//================================================================
  int GUI_mList_get_sel_nr (GUIobj_mList *wLst) {
//================================================================
/// GUI_mList_get_sel_nr   get lineNr of selected line

  int          lNr;
  GtkTreeIter  iter;
  GtkTreeModel *model;


  gtk_tree_selection_get_selected (wLst->sel, &model, &iter);

  lNr = GUI_mList_iter2rowNr (wLst, &iter);


  return lNr;

}


//================================================================
  int GUI_mList__ (GtkWidget *parent,
                   GUIobj_mList *nLst,
                   char *hdrLn, int iSel,
                   void *selCB, void *keyCB, void *mouseCB) {
//================================================================
/// \code
/// create simple List (1 column only)
///   hdrLn  title in headerline;  NULL = no headerline.
///   iSel   0=not selectable, 1=singleSelections, 2=multiSelections
///   wSiz   unused
/// \endcode

  int               ii;
  GtkWidget         *ws;
  GtkCellRenderer   *renderer;
  GtkTreeViewColumn *column;

  GtkTreeModel      **w_tree;
  GtkWidget         **w_list;
  GtkTreeSelection  **w_sel;


  //----------------------------------------------------------------

  w_tree = &(nLst->tree);
  w_list = &(nLst->list);
  w_sel  = &(nLst->sel);



  // create adjustments for tree
  ws = gtk_scrolled_window_new (NULL, NULL);
  gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (ws),
         GTK_SHADOW_ETCHED_IN);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (ws),
                                  GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
                                  // GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
  // gtk_box_pack_start (GTK_BOX (parent), ws, FALSE, TRUE, 0);
  gtk_box_pack_start (GTK_BOX (parent), ws, TRUE, TRUE, 0);



  // create List   (anzahl, typ1, typ2, ...);
  *w_list = (GtkWidget*)gtk_list_store_new (1, G_TYPE_STRING);  // 1. col.
                                    // G_TYPE_STRING,     // 2. col.
                                    // G_TYPE_STRING);    // 3. col


  *w_tree = (GtkTreeModel*)gtk_tree_view_new_with_model (
                 GTK_TREE_MODEL (*w_list));
  // gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (*w_tree), TRUE);
  // gtk_tree_view_set_fixed_height_mode (GTK_TREE_VIEW (*w_tree), TRUE);
  // gtk_tree_view_set_search_column (GTK_TREE_VIEW (*w_tree), 0);
  g_object_set (G_OBJECT(*w_tree), "fixed-height-mode", TRUE, NULL);



  g_object_unref (*w_list);


  gtk_container_add (GTK_CONTAINER (ws), (GtkWidget*)*w_tree);


  // iSizY = UI_fontsizY + (UI_fontsizY / 2);  // was 4
  // iSizY = UI_fontsizY - 5;
    // printf(" iSizY=%d\n",iSizY);


  // erste Spalte
  renderer = gtk_cell_renderer_text_new ();
  // g_signal_connect (renderer, "toggled",
                    // G_CALLBACK (fixed_toggled), *w_list);
  // g_object_set (G_OBJECT (renderer), "foreground", "black", NULL);
  g_object_set (G_OBJECT (renderer), "ypad", 0, NULL);  // y-offset
  // g_object_set (G_OBJECT(renderer), "height", 5, NULL);
  // g_object_set (G_OBJECT(renderer), "sensitive", FALSE, NULL);
  // do not enable modification of cell ..  GEHT NED !
  // g_object_set (G_OBJECT(renderer),"mode",GTK_CELL_RENDERER_MODE_INERT,NULL);
  // g_object_set (G_OBJECT(renderer),"mode",GTK_CELL_RENDERER_MODE_ACTIVATABLE,NULL);


  column = gtk_tree_view_column_new_with_attributes (hdrLn, 
                            renderer,
                            "text", 0,  // typ, spaltenNr
                            NULL);      // terminator

  // gtk_tree_view_column_set_fixed_width (column, 22);
  // gtk_tree_view_column_set_spacing (column, 0);
  g_object_set (G_OBJECT (column), "sizing", GTK_TREE_VIEW_COLUMN_FIXED, NULL);


  // gtk_tree_view_column_set_attributes (column, renderer,
                           // "sizing", GTK_TREE_VIEW_COLUMN_FIXED,
                           // NULL);      // terminator


  gtk_tree_view_append_column (GTK_TREE_VIEW (*w_tree), column);

  // gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW (ws),
                                             // (GtkWidget*)*w_tree);
  // gtk_widget_show_all (ws);





  // Setup the selection handler
  *w_sel = gtk_tree_view_get_selection (GTK_TREE_VIEW (*w_tree));
  if(iSel == 0) ii = GTK_SELECTION_NONE;
  else if(iSel == 1) ii = GTK_SELECTION_SINGLE;
  else if(iSel == 2) ii = GTK_SELECTION_MULTIPLE;
  gtk_tree_selection_set_mode (*w_sel, ii);
  


  if(selCB) {
      g_signal_connect (G_OBJECT (*w_sel), "changed",
                        G_CALLBACK (selCB),
                        NULL);
  }


      // connect KeybordEvents in MainWindow
  if(keyCB) {
      // es geht nicht: *w_list ws 
      gtk_signal_connect (GTK_OBJECT(*w_tree),
                         "key_press_event",
                         GTK_SIGNAL_FUNC (keyCB), NULL);
  }

      // connect Mausbuttons in MainWindow
  if(mouseCB) {
      // es geht nicht: 
      gtk_signal_connect (GTK_OBJECT(*w_tree),
                         "button_press_event",
                         GTK_SIGNAL_FUNC (mouseCB), NULL);
  }


  
  // disable Headerline
  if(hdrLn == NULL)
  gtk_tree_view_set_headers_visible (GTK_TREE_VIEW(*w_tree), FALSE);




  // gtk_widget_set_usize (w_list, 500, 0);

  return 0;

}


// EOF
