/*  ut_wm_x11.c          Windowmanagerfunctions                   2009-03-11

    copyright            : (C) 2009 by CADCAM-Services Franz Reiter
    email                : franz.reiter@cadcam.co.at

****************************************************************************



-------------------------------------------------------------------------
Liste_Funktionen_Beginn:

WM_raise_wTit              raise window with title <wTit>

WM_open                    open wm
WM_resWin0                 resolv first-level-windowTree
WM_resWin1                 resolv second-level-windowTree

Liste_Funktionen_Ende:
-------------------------------------------------------------------------

xwininfo -root -tree

XGetTextproperty ... noch nicht in X11.R1
XWMHints S296 (WindowmanagerHints)   XGetWMHints



*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>


#include <X11/Xlib.h>
#include <X11/Xatom.h>           // XA_WM_COMMAND XA_WM_NAME XA_STRING
#include <X11/Xutil.h>           // XSizeHints




/*
//================================================================
   int WM_getBuffer (char *txt, int maxSiz) {
//================================================================
// WM_getBuffer               get buffercontent

  int     i1, bNr;
  Display *dpy;
  char    *cb;

  WM_open (&dpy);
  if(!dpy) return -1;

  for(bNr=0; bNr<8; ++bNr) {
    // buffer#0-7; must not contain text ..
    cb = XFetchBuffer (dpy, &i1, bNr);  // S.169
      printf(" XFetchBuffer[%d] %d |%s|\n",bNr,i1,cb);
    if(i1 > 0) break;
  }


  if(i1 >= maxSiz) i1 = maxSiz;
  strncpy(txt, cb, i1);
  txt[i1] = '\0';

  XFree (cb);

  XCloseDisplay (dpy);

  return 0;

}
*/

///================================================================
  int WM_open (Display **dpy) {
///================================================================

  char    *dpy_name;

  if ((dpy_name = getenv ("DISPLAY")) == NULL) dpy_name = NULL;   // ":0.0"

  *dpy = XOpenDisplay (dpy_name);
  if (*dpy == NULL) {
    printf("****** ERROR XOpenDisplay\n");
    return -2;
  }

  return 0;

}


///================================================================
  int WM_raise_wTit (char *wTit) {
///================================================================
/// WM_raise_wTit              raise window with title <wTit>
/// RetCod:
///    0     OK, window exists, raised
///   -1     window with title <wTit> does not exist
///   -2     Error accessing windowmanager
 
  int     irc;
  Display *dpy;
  Window  win;

  printf("WM_raise_wTit |%s|\n",wTit);


  WM_open (&dpy);
  if(!dpy) return -2;


  // XGrabServer(dpy);

  irc = WM_resWin0 (&win, dpy, wTit);

  if(irc >= 0) {
    printf(" raise %x\n",win);
    // XCirculateSubwindowsUp (dpy, win);  // S.76
    XUnmapWindow (dpy, win);
    XMapRaised (dpy, win);
    //XMapWindow (dpy, win);
    //XRaiseWindow (dpy, win);           // S.337
    XFlush (dpy);
  }
  XSync (dpy, 0);

  // XUngrabServer(dpy);


  XCloseDisplay (dpy);

  return irc;

}


//================================================================
  int WM_resWin0 (Window *wino, Display *dpy, char *wTit) {
//================================================================
// resolv first-level-windowTree
// RetCod:
//    0     OK, window <wTit> exists; it is <wino>.
//   -1     window with title <wTit> does not exist

  int     irc, i1, cNr;
  Window  win, winr, winp, *winc;


  win = DefaultRootWindow(dpy);

  irc = XQueryTree (dpy, win, &winr, &winp, &winc, &cNr);      // S336
  if(irc != 1) {
    printf("***** Error XQueryTree %d\n",irc);
    return -1;
  }


  // printf(" cNr=%d\n",cNr);

  for(i1=0; i1<cNr; ++i1) {
      // printf("========= %d %x =========\n",i1,winc[i1]);

    // resolv next level
    irc = WM_resWin1 (wino, dpy, &winc[i1], wTit);
    if(irc == 0) {
        // printf(" - found0 %x\n",winc[i1]);
      // *wino = winc[i1];
      break;
    }
  }

  XFree (winc);

  return irc;

}
 

//======================================================================
  int WM_resWin1 (Window *wino, Display *dpy, Window *wini, char *wTit) {
//======================================================================
// resolv second-level-windowTree
// RetCod:
//    0     OK, window <wTit> exists; it is <wino>.
//   -1     window with title <wTit> does not exist
 
  int     irc, i1, cNr;
  Window  winr, winp, *winc;
  char    *cp1;

  //XSizeHints  wmsh;
  //XWindowAttributes wmwa;

  // int      act_format;
  // Atom     act_type;
  // unsigned long l1, l2;


  // get all childWindows of win
  irc = XQueryTree (dpy, *wini, &winr, &winp, &winc, &cNr);     // S.336; 0=Err.
  if(irc != 1) {
    printf("***** Error XQueryTree %d\n",irc);
    return -1;
  }

  if(cNr < 1) return -1;   // continue ..


    // printf("WM_resWin1 cNr=%d\n",cNr);

  for(i1=0; i1<cNr; ++i1) {
      // printf("  WM_resWin1 %d %x\n",i1,winc[i1]);

    //// skip window without SizeHints - geht nicht mit KDE !
    //irc = XGetNormalHints (dpy, winc[i1], &wmsh);  // 0=Err
    //if(irc == 0) continue;
      // printf(" XGetNormalHints %d x=%d y=%d w=%d h=%d\n",irc,
               // wmsh.x,wmsh.y,wmsh.width,wmsh.height);

    irc = XFetchName (dpy, winc[i1], &cp1);   // get XA_WM_NAME S.171; 
    if(irc != 0) {
      // printf(" name[%d] = |%s|\n",i1,cp1);

      if(!strcmp(cp1, wTit)) {
          printf("  - found1 %x\n",winc[i1]);
        *wino = winc[i1];
        XFree (cp1);
        XFree (winc);
        return 0;
      }

      XFree (cp1);

    }

/*
    irc = XGetWindowAttributes (dpy, winc[i1], &wmwa);  // 0=Err
      printf(" XGetWindowAttributes %d x=%d y=%d w=%d h=%d s=%d\n",irc,
               wmwa.x,wmwa.y,wmwa.width,wmwa.height,wmwa.map_state);

    irc = XGetWindowProperty (dpy,  winc[i1],               // S241; 0=OK; 1=Err
            XA_WM_COMMAND,        // XA_WM_COMMAND, XA_WM_NAME
            0,                    // ioffset,
            1024L,                // length to retrieve
            False,                // delete,
            XA_STRING,            // reqired typ, XA_STRING,
            &act_type,            // actual type
            &act_format,
            &l1,                  // i2,
            &l2,                  // ibytes,
            &cp1);                // property

      printf("  irc=%d cmd[%d]=|%s|\n",irc,i1,cp1);
    XFree (cp1);
*/

    irc = WM_resWin1 (wino, dpy, &winc[i1], wTit);
    if(irc == 0) return 0;

  }

  XFree (winc);


  return -1;

}


// EOF

